# Copyright (C) 2013-2020 Sascha Manns <Sascha.Manns@outlook.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Dependencies

require 'hoe'

# rubocop:disable Style/ClassAndModuleChildren
module Hoe::Packaging
  # Versionizing
  VERSION = '1.2.7'.freeze
  attr_accessor :create_packages
  attr_accessor :deploy_packages

  # Initialize plugin
  def initialize_packaging
    require 'fileutils'
    require 'parseconfig'
    require 'rainbow/ext/string'
    require 'fpm'
    require 'yaml'
  end

  def define_packaging_tasks
    # Rake Task for building packages
    desc 'Creating deb and rpm files'
    task :create_packages do
      create_packages_method
    end

    # Rake task for deploying packages
    desc 'Deploying packages to bintray'
    task :deploy_packages do
      deploy_bintray_method
    end
  end

  # Method for creating deb and rpm packages
  # @return [String] true or false
  def create_packages_method
    FileUtils.cd('recipes') do
      puts 'Creating the deb package'.color(:yellow)
      system('fpm-cook -t deb')
      puts 'deb creating done'.color(:green)
      puts 'Creating the rpm package'.color(:yellow)
      system('fpm-cook -t rpm')
      puts 'rpm creating done'.color(:green)
    end
  end

  # Method for deploying to bintray
  # rubocop:disable Metrics/AbcSize
  # rubocop:disable Metrics/LineLength
  # rubocop:disable Metrics/MethodLength
  # This method smells of :reek:TooManyStatements
  # @return [String] true or false
  def deploy_bintray_method
    project = projectname
    user, apikey = bintray_config
    version = version
    path = 'pool/main/r'
    debadd = ';deb_distribution=ubuntu;deb_component=main;deb_architecture=i386,amd64'
    publish = ';publish=1'
    puts 'Deploying packages to bintray'.color(:yellow)

    FileUtils.cd('recipes/pkg') do
      filerpm = Dir.glob('*.rpm')
      filedeb = Dir.glob('*.deb')
      rpm = filerpm.first.to_s
      deb = filedeb.first.to_s

      rpmup = system("curl -T #{rpm} -u#{user}:#{apikey} https://api.bintray.com/content/#{user}/rpm/#{project}/v#{version}/#{path}/#{publish}")
      begin
        rpmup['message'] == 'success'
      rescue NoMethodError
        false
      end
      debup = system("curl -T #{deb} -u#{user}:#{apikey} 'https://api.bintray.com/content/#{user}/deb/#{project}/v#{version}/#{path}/#{deb}#{debadd}#{publish}'")
      begin
        debup['message'] == 'success'
      rescue NoMethodError
        false
      end
    end
    puts 'Deploying succeeded'.color(:green)
  end

  # Method for getting the project name
  # @return [String] project Returns project name
  def projectname
    pnameraw = File.open(*Dir.glob('README.*'), &:readline)
    project = pnameraw.gsub(/[^0-9A-Za-z_-]/, '')
    return project
  end

  # Method for getting the bintray config
  # @return [Array] user apikey
  def bintray_config
    config = YAML.safe_load(File.read("#{Dir.home}/.hoerc"))
    user = config['deploy']['username'].to_s
    apikey = config['deploy']['apikey'].to_s
    [user, apikey]
  end

  # Method for getting version
  # @return [String] version
  def version
    version = File.open(*Dir.glob('VERSION'), &:readline)
    version.chomp.to_s
  end
end
